from dataclasses import dataclass
import logging
import tomli
from eunice.definitions import CONFIG_FILE_LOCATION
from eunice.utils import check_for_file

logger = logging.getLogger(__name__)
  

@dataclass
class Config:
    openai_secret_key=None


class ConfigFile:
    def __init__(self):
        self.config_file = CONFIG_FILE_LOCATION
        self.is_config = check_for_file(self.config_file)


    def load_config(self):
        with open(self.config_file, mode='rb') as f:
            return tomli.load(f)

    def set_config(self, config: Config):

        if self.is_config is True:
            config_file = self.load_config()
            logger.info(config_file)
            try:
                general = config_file["general"]
                try:
                    config.openai_secret_key = general["openai_secret_key"]
                    logger.info(f"CONFIG: Found openai secret key!")
                except KeyError as k:
                    print("CONFIG ERROR: Missing OpenAI Secret Key")
                
            except KeyError as k:
                logger.info(k)
                pass



import argparse


def get_arguments():
    parser = argparse.ArgumentParser(
        description="Eunice - CLI wrapper for ChatGPT"
    )
    parser.add_argument(
        "-l",
        "--log",
        help="Set the log level",
        type=str,
        choices=[
            "DEBUG",
            "INFO",
        ],
    )
    return parser.parse_args()



import os
import logging


logger = logging.getLogger(__name__)


def check_for_dir(path):
    is_dir = os.path.isdir(path)
    logger.info(f"check_for_dir {path} {is_dir}")
    return is_dir

def check_for_file(path):
    is_file = os.path.isfile(path)
    logger.info(f"check_for_file {path} {is_file}")
    return is_file



from pathlib import Path


HOME_DIR = home = str(Path.home())
CONFIG_DIR_NAME = '.eunice'
CONFIG_FILE_NAME = 'eunice.cfg'
CONFIG_FILE_LOCATION = f"{HOME_DIR}/{CONFIG_DIR_NAME}/{CONFIG_FILE_NAME}"
EUNICE_HOME = f"{HOME_DIR}/{CONFIG_DIR_NAME}"
HOME_DIR_CREATE = f"mkdir {EUNICE_HOME}"



import openai
import sys
import subprocess
import logging
from eunice.models import Config, ConfigFile
from eunice.definitions import HOME_DIR_CREATE, EUNICE_HOME
from eunice.utils import check_for_dir

class bcolors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKCYAN = '\033[96m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'


logger = logging.getLogger(__name__)

def config_allocation(config):
    config_file = ConfigFile()
    config_file.set_config(config)

def run_subprocess(script: str):
    if "rm " in script:
        sys.exit()
    try:
        subprocess.run(script, shell=True, check=True, timeout=30)
    except subprocess.CalledProcessError as e:
        logger.error(e)

def virgin_boostrap():
    global initial_boot_flag
    initial_boot_flag = True
    logger.info("First boot")
    run_subprocess(HOME_DIR_CREATE)

def chat_create(message_history: list[dict]):
    return openai.ChatCompletion.create(model="gpt-3.5-turbo", messages=message_history)

def ask(config: Config):

    if check_for_dir(EUNICE_HOME) is False:
        virgin_boostrap()

    config_allocation(config)
    openai.api_key = config.openai_secret_key

    print(bcolors.HEADER + """
███████╗██╗   ██╗███╗   ██╗██╗ ██████╗███████╗
██╔════╝██║   ██║████╗  ██║██║██╔════╝██╔════╝
█████╗  ██║   ██║██╔██╗ ██║██║██║     █████╗  
██╔══╝  ██║   ██║██║╚██╗██║██║██║     ██╔══╝  
███████╗╚██████╔╝██║ ╚████║██║╚██████╗███████╗
╚══════╝ ╚═════╝ ╚═╝  ╚═══╝╚═╝ ╚═════╝╚══════╝                          
    """ + bcolors.ENDC)
    print(bcolors.WARNING + "Press Ctrl-C at any time to quit." + bcolors.ENDC)

    message_history = []

    prep_for_nan = {"role": "system", "content": "You are a friendly assistance called Eunice."}
    message_history.append(prep_for_nan)
    print(bcolors.OKGREEN + "Eunice here, how can I help you?" + bcolors.ENDC)
    try:
        while True:
            user_chat = input(bcolors.OKCYAN + ">> " + bcolors.ENDC)
            assemble_message = {"role": "system", "content": user_chat}
            message_history.append(assemble_message)
            chat_completion = chat_create(message_history)
            print(bcolors.OKGREEN + chat_completion.choices[0].message.content + bcolors.ENDC)
    except KeyboardInterrupt:
        print(bcolors.WARNING + "\nQuitting.." + bcolors.ENDC)
        sys.exit()




from setuptools import setup


VERSION = "0.0.3"


with open("README.md") as f:
    readme = f.read()


setup(
    name="eunice",
    version=VERSION,
    description="eunice - a simple chatgpt cli wrapper",
    long_description=readme,
    long_description_content_type="text/markdown",
    url="http://github.com/rob-parker-what/eunice",
    author="rob-parker-what",
    author_email="robparkerwhat.dev@gmail.com",
    license="MIT",
    packages=["eunice"],
    entry_points={
        "console_scripts": ["eunice=eunice.cmdline:main"],
    },
    install_requires=[
        "openai",
        "tomli"
    ],
    zip_safe=False,
    python_requires=">=3.8",
    keywords="chatgpt, ai, openai",
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Topic :: Software Development",
        "Operating System :: Unix",
        "Operating System :: MacOS",
    ],
)


from eunice.run import ask
from eunice.models import Config
from eunice.args import get_arguments
from eunice.eunice_logging import logging_config


config = Config()

def main():
    args = get_arguments()
    logging_config(args.log)
    ask(config)


if __name__ == '__main__':
    main()
